@@ -18,28 +18,47 @@
 using System;
 using System.IO;
 using System.Collections.Generic;
-using System.Xml;
 using System.Security.Principal;
-
-//using Mono.Unix;
+using System.Xml;
 
 namespace SparkleLib {
 
     public class SparkleConfig : XmlDocument {
 
-        public static string ConfigPath = Path.Combine (
+        private static string default_config_path = Path.Combine (
             Environment.GetFolderPath (Environment.SpecialFolder.ApplicationData),
                 "sparkleshare");
 
-        public static SparkleConfig DefaultConfig = new SparkleConfig (ConfigPath, "config.xml");
+        // TODO: declare elsewhere
+        public static SparkleConfig DefaultConfig = new SparkleConfig (default_config_path, "config.xml");
+        public static bool DebugMode = true;
+
+
         public string FullPath;
+        public string TmpPath;
+        public string LogFilePath;
 
         public string HomePath {
             get {
-		        return Environment.GetFolderPath (Environment.SpecialFolder.Personal);
-		    }
-        }
+                if (GetConfigOption ("home_path") != null) {
+                    return GetConfigOption ("home_path");
 
+                } else if (SparkleHelpers.IsWindows) {
+                    try {
+                        Environment.SpecialFolder folder =
+                            (Environment.SpecialFolder) Enum.Parse (
+                                typeof(Environment.SpecialFolder), "UserProfile");
+
+                        return (Environment.GetFolderPath (folder));
+
+                    } catch {
+                        return Environment.GetFolderPath (Environment.SpecialFolder.Personal);
+                    }
+                } else
+                    return Environment.GetFolderPath (Environment.SpecialFolder.Personal);
+
+            }
+        }
 
         public string FoldersPath {
             get {
@@ -50,31 +69,53 @@ namespace SparkleLib {
             }
         }
 
-        public string TmpPath {
-            get {
-                return Path.Combine (FoldersPath, ".tmp");
-            }
-        }
-
-
         public SparkleConfig (string config_path, string config_file_name)
         {
-            FullPath = System.IO.Path.Combine (config_path, config_file_name);
+            try {
+                Environment.SpecialFolder folder =
+                    (Environment.SpecialFolder) Enum.Parse(
+                                typeof(Environment.SpecialFolder), "UserProfile");
 
-            if (!Directory.Exists (config_path)) {
+                string old_path = Path.Combine (
+                    Environment.GetFolderPath (Environment.SpecialFolder.Personal), "SparkleShare");
+
+                if (Directory.Exists (old_path) &&
+                    Environment.OSVersion.Platform == PlatformID.Win32NT) {
+
+                    string new_path = Path.Combine (Environment.GetFolderPath (folder), "SparkleShare");
+                    Directory.Move (old_path, new_path);
+
+                    Console.WriteLine ("Migrated SparkleShare folder to %USERPROFILE%");
+
+                }
+
+            } catch (Exception e) {
+                Console.WriteLine ("Failed to migrate: " + e.Message);
+                // TODO: Remove this block when most people have migrated to the new path
+            }
+
+            FullPath    = Path.Combine (config_path, config_file_name);
+            LogFilePath = Path.Combine (config_path, "debug.log");
+
+            if (File.Exists (LogFilePath)) {
+                try {
+                    File.Delete (LogFilePath);
+
+                } catch (Exception) {
+                    // Don't delete the debug.log if 'tail' is reading it
+                }
+            }
+
+            if (!Directory.Exists (config_path))
                 Directory.CreateDirectory (config_path);
-                SparkleHelpers.DebugInfo ("Config", "Created \"" + config_path + "\"");
-            }
 
-            string icons_path = System.IO.Path.Combine (config_path, "icons");
-            if (!Directory.Exists (icons_path)) {
+            string icons_path = Path.Combine (config_path, "icons");
+            if (!Directory.Exists (icons_path))
                 Directory.CreateDirectory (icons_path);
-                SparkleHelpers.DebugInfo ("Config", "Created \"" + icons_path + "\"");
-            }
 
             try {
               Load (FullPath);
-              
+
             } catch (TypeInitializationException) {
                 CreateInitialConfig ();
 
@@ -82,9 +123,9 @@ namespace SparkleLib {
                 CreateInitialConfig ();
 
             } catch (XmlException) {
-            
+
                 FileInfo file = new FileInfo (FullPath);
-                
+
                 if (file.Length == 0) {
                     File.Delete (FullPath);
                     CreateInitialConfig ();
@@ -95,9 +136,8 @@ namespace SparkleLib {
 
             } finally {
                 Load (FullPath);
+                TmpPath = Path.Combine (FoldersPath, ".tmp");
             }
-
-            ConfigureSSH ();
         }
 
 
@@ -130,8 +170,6 @@ namespace SparkleLib {
                 "    <email>Unknown</email>" + n +
                 "  </user>" + n +
                 "</sparkleshare>");
-
-            SparkleHelpers.DebugInfo ("Config", "Created \"" + FullPath + "\"");
         }
 
 
@@ -143,7 +181,17 @@ namespace SparkleLib {
                 XmlNode email_node = SelectSingleNode ("/sparkleshare/user/email/text()");
                 string email = email_node.Value;
 
-                return new SparkleUser (name, email);
+                string pubkey_file_path = Path.Combine (
+                    Path.GetDirectoryName (FullPath),
+                    "sparkleshare." + email + ".key.pub"
+                );
+
+                SparkleUser user = new SparkleUser (name, email);
+
+                if (File.Exists (pubkey_file_path))
+                    user.PublicKey = File.ReadAllText (pubkey_file_path);
+
+                return user;
             }
 
             set {
@@ -155,13 +203,18 @@ namespace SparkleLib {
                 XmlNode email_node = SelectSingleNode ("/sparkleshare/user/email/text()");
                 email_node.InnerText = user.Email;
 
-                this.Save ();
+                Save ();
+
+                // ConfigureSSH ();
             }
         }
 
-
+/*
         private void ConfigureSSH ()
         {
+            if (User.Email.Equals ("Unknown"))
+                return;
+
             string path = Environment.GetFolderPath (Environment.SpecialFolder.Personal);
 
             if (!(SparkleBackend.Platform == PlatformID.Unix ||
@@ -172,8 +225,14 @@ namespace SparkleLib {
 
             string ssh_config_path      = Path.Combine (path, ".ssh");
             string ssh_config_file_path = SparkleHelpers.CombineMore (path, ".ssh", "config");
-            string ssh_config           = "IdentityFile " +
-                Path.Combine (SparkleConfig.ConfigPath, "sparkleshare." + User.Email + ".key");
+
+            string ssh_key_path = SparkleHelpers.NormalizeSeparatorsToOS(
+                Path.Combine(SparkleConfig.ConfigPath, "sparkleshare." + User.Email + ".key"));
+            if (SparkleHelpers.IsWindows && ssh_key_path.IndexOf(' ') >= 0)
+            {
+                ssh_key_path = "\"" + ssh_key_path + "\"";
+            }
+            string ssh_config = "IdentityFile " + ssh_key_path;
 
             if (!Directory.Exists (ssh_config_path))
                 Directory.CreateDirectory (ssh_config_path);
@@ -198,13 +257,11 @@ namespace SparkleLib {
                 File.WriteAllText (ssh_config_file_path, ssh_config);
             }
 
-            //UnixFileSystemInfo file_info = new UnixFileInfo (ssh_config_file_path);
-            //file_info.FileAccessPermissions = (FileAccessPermissions.UserRead |
-            //                                   FileAccessPermissions.UserWrite); TODO
+            Chmod644 (ssh_config_file_path);
 
             SparkleHelpers.DebugInfo ("Config", "Added key to " + ssh_config_file_path);
         }
-
+*/
 
         public List<string> Folders {
             get {
@@ -237,7 +294,7 @@ namespace SparkleLib {
             XmlNode node_root = SelectSingleNode ("/sparkleshare");
             node_root.AppendChild (node_folder);
 
-            this.Save ();
+            Save ();
         }
 
 
@@ -248,32 +305,32 @@ namespace SparkleLib {
                     SelectSingleNode ("/sparkleshare").RemoveChild (node_folder);
             }
 
-            this.Save ();
+            Save ();
         }
 
 
         public bool FolderExists (string name)
         {
-            XmlNode folder = this.GetFolder (name);
+            XmlNode folder = GetFolder (name);
             return (folder != null);
         }
 
 
         public string GetBackendForFolder (string name)
         {
-            return this.GetFolderValue (name, "backend");
+            return GetFolderValue (name, "backend");
         }
 
 
         public string GetUrlForFolder (string name)
         {
-            return this.GetFolderValue (name, "url");
+            return GetFolderValue (name, "url");
         }
 
 
         public bool SetFolderOptionalAttribute (string folder_name, string key, string value)
         {
-            XmlNode folder = this.GetFolder (folder_name);
+            XmlNode folder = GetFolder (folder_name);
 
             if (folder == null)
                 return false;
@@ -287,13 +344,14 @@ namespace SparkleLib {
                 folder.AppendChild (new_node);
             }
 
+            Save ();
             return true;
         }
 
 
         public string GetFolderOptionalAttribute (string folder_name, string key)
         {
-            XmlNode folder = this.GetFolder (folder_name);
+            XmlNode folder = GetFolder (folder_name);
 
             if (folder != null) {
                 if (folder [key] != null)
@@ -353,8 +411,8 @@ namespace SparkleLib {
 
         private string GetFolderValue (string name, string key)
         {
-            XmlNode folder = this.GetFolder(name);
-            
+            XmlNode folder = GetFolder(name);
+
             if ((folder != null) && (folder [key] != null)) {
                 return folder [key].InnerText;
             }
@@ -390,7 +448,7 @@ namespace SparkleLib {
             }
 
             SparkleHelpers.DebugInfo ("Config", "Updated " + name + ":" + content);
-            this.Save ();
+            Save ();
         }
 
 
@@ -399,9 +457,19 @@ namespace SparkleLib {
             if (!File.Exists (FullPath))
                 throw new ConfigFileNotFoundException (FullPath + " does not exist");
 
-            this.Save (FullPath);
+            Save (FullPath);
             SparkleHelpers.DebugInfo ("Config", "Updated \"" + FullPath + "\"");
         }
+
+
+        private void Chmod644 (string file_path)
+        {
+            // Hack to be able to set the permissions on a file
+            // that OpenSSH still likes without resorting to Mono.Unix
+            FileInfo file_info   = new FileInfo (file_path);
+            file_info.Attributes = FileAttributes.ReadOnly;
+            file_info.Attributes = FileAttributes.Normal;
+        }
     }
 
 
