@@ -16,14 +16,11 @@
 
 
 using System;
-using System.Collections.Generic;
-using System.Diagnostics;
-using System.IO;
-using System.Runtime.InteropServices;
-using System.Text;
+using System.Threading;
 
+#if __MonoCS__
 using Mono.Unix;
-//using Mono.Unix.Native;
+#endif
 using SparkleLib;
 
 namespace SparkleShare {
@@ -33,15 +30,24 @@ namespace SparkleShare {
 
         public static SparkleController Controller;
         public static SparkleUI UI;
-
-
+		
+		public static Mutex ProgramMutex = new Mutex (false, "SparkleShare");
+		
+		
         // Short alias for the translations
         public static string _ (string s)
         {
+        #if __MonoCS__
             return Catalog.GetString (s);
+         #else
+            return Strings.T (s);
+         #endif
         }
         
-
+     
+        #if !__MonoCS__
+        [STAThread]
+        #endif
         public static void Main (string [] args)
         {
             // Parse the command line options
@@ -63,7 +69,14 @@ namespace SparkleShare {
             if (show_help)
                 ShowHelp (option_set);
 
-
+			
+			// Only allow one instance of SparkleShare
+			if (!ProgramMutex.WaitOne (0, false)) {
+				Console.WriteLine ("SparkleShare is already running.");
+				Environment.Exit (-1);
+			}
+				
+			
             // Initialize the controller this way so that
             // there aren't any exceptions in the OS specific UI's
             Controller = new SparkleController ();
@@ -73,6 +86,12 @@ namespace SparkleShare {
                 UI = new SparkleUI ();
                 UI.Run ();
             }
+         
+            #if !__MonoCS__
+            // Suppress assertion messages in debug mode
+            GC.Collect (GC.MaxGeneration, GCCollectionMode.Forced);
+            GC.WaitForPendingFinalizers ();
+            #endif
         }
 
 
@@ -88,10 +107,10 @@ namespace SparkleShare {
             Console.WriteLine (_("This is free software, and you are welcome to redistribute it "));
             Console.WriteLine (_("under certain conditions. Please read the GNU GPLv3 for details."));
             Console.WriteLine (" ");
-            Console.WriteLine (_("SparkleShare automatically syncs Git repositories in "));
-            Console.WriteLine (_("the ~/SparkleShare folder with their remote origins."));
+            Console.WriteLine (_("SparkleShare is a collaboration and sharing tool that is "));
+            Console.WriteLine (_("designed to keep things simple and to stay out of your way."));
             Console.WriteLine (" ");
-            Console.WriteLine (_("Usage: sparkleshare [start|stop|restart] [OPTION]..."));
+            Console.WriteLine (_("Usage: sparkleshare [start|stop|restart|version] [OPTION]..."));
             Console.WriteLine (_("Sync SparkleShare folder with remote repositories."));
             Console.WriteLine (" ");
             Console.WriteLine (_("Arguments:"));
@@ -101,30 +120,10 @@ namespace SparkleShare {
         }
 
 
-        // Prints the version information
         public static void PrintVersion ()
         {
             Console.WriteLine (_("SparkleShare " + Defines.VERSION));
             Environment.Exit (0);
         }
-
-
-        // Strange magic needed by SetProcessName ()
-//        [DllImport ("libc")]
-//        private static extern int prctl (int option, byte [] arg2, IntPtr arg3, IntPtr arg4, IntPtr arg5);
-        
-        
-        // Sets the Unix process name to 'sparkleshare' instead of 'mono'
-/*        private static void SetProcessName (string name)
-        {
-            try {
-                if (prctl (15, Encoding.ASCII.GetBytes (name + "\0"), IntPtr.Zero, IntPtr.Zero, IntPtr.Zero) != 0)
-                    throw new ApplicationException ("Error setting process name: " +
-                                                    Mono.Unix.Native.Stdlib.GetLastError ());
-
-            } catch (EntryPointNotFoundException) {
-                Console.WriteLine ("SetProcessName: Entry point not found");
-            }
-        } */
     }
 }
