@@ -28,6 +28,12 @@ namespace SparkleShare {
 
     public class SparkleEventLogController {
 
+        public event ShowWindowEventHandler ShowWindowEvent;
+        public delegate void ShowWindowEventHandler ();
+
+        public event HideWindowEventHandler HideWindowEvent;
+        public delegate void HideWindowEventHandler ();
+
         public event UpdateContentEventEventHandler UpdateContentEvent;
         public delegate void UpdateContentEventEventHandler (string html);
 
@@ -54,6 +60,9 @@ namespace SparkleShare {
                 if (ContentLoadingEvent != null)
                     ContentLoadingEvent ();
 
+                if (UpdateSizeInfoEvent != null)
+                    UpdateSizeInfoEvent ("…", "…");
+
                 Stopwatch watch = new Stopwatch ();
                 watch.Start ();
 
@@ -63,8 +72,10 @@ namespace SparkleShare {
 
                     // A short delay is less annoying than
                     // a flashing window
-                    if (watch.ElapsedMilliseconds < 500)
-                        Thread.Sleep (500 - (int) watch.ElapsedMilliseconds);
+					int delay = 1000;
+					
+                    if (watch.ElapsedMilliseconds < delay)
+                        Thread.Sleep (delay - (int) watch.ElapsedMilliseconds);
 
                     if (UpdateContentEvent != null)
                         UpdateContentEvent (html);
@@ -80,7 +91,13 @@ namespace SparkleShare {
         public string HTML {
             get {
                 List<SparkleChangeSet> change_sets = Program.Controller.GetLog (this.selected_folder);
-                return Program.Controller.GetHTMLLog (change_sets);
+
+                string html = Program.Controller.GetHTMLLog (change_sets);
+
+                if (UpdateSizeInfoEvent != null)
+                    UpdateSizeInfoEvent (Size, HistorySize);
+
+                return html;
             }
         }
 
@@ -97,11 +114,18 @@ namespace SparkleShare {
                 foreach (SparkleRepoBase repo in Program.Controller.Repositories) {
                     if (this.selected_folder == null)
                         size += repo.Size;
-                    else if (this.selected_folder.Equals (repo.Name))
-                        return Program.Controller.FormatSize (repo.Size);
+                    else if (this.selected_folder.Equals (repo.Name)) {
+                        if (repo.Size == 0)
+                            return "???";
+                        else
+                            return Program.Controller.FormatSize (repo.Size);
+                    }
                 }
 
-                return Program.Controller.FormatSize (size);
+                if (size == 0)
+                    return "???";
+                else
+                    return Program.Controller.FormatSize (size);
             }
         }
 
@@ -112,30 +136,42 @@ namespace SparkleShare {
                 foreach (SparkleRepoBase repo in Program.Controller.Repositories) {
                     if (this.selected_folder == null)
                         size += repo.HistorySize;
-                    else if (this.selected_folder.Equals (repo.Name))
-                        return Program.Controller.FormatSize (repo.HistorySize);
+                    else if (this.selected_folder.Equals (repo.Name)) {
+                        if (repo.HistorySize == 0)
+                            return "???";
+                        else
+                            return Program.Controller.FormatSize (repo.HistorySize);
+                    }
                 }
 
-                return Program.Controller.FormatSize (size);
+                if (size == 0)
+                    return "???";
+                else
+                    return Program.Controller.FormatSize (size);
             }
         }
 
 
         public SparkleEventLogController ()
         {
+            Program.Controller.ShowEventLogWindowEvent += delegate {
+                if (ShowWindowEvent != null)
+                    ShowWindowEvent ();
+            };
+
             Program.Controller.AvatarFetched += delegate {
                 if (UpdateContentEvent != null)
                     UpdateContentEvent (HTML);
             };
-
+			
             Program.Controller.OnIdle += delegate {
                 if (UpdateContentEvent != null)
                     UpdateContentEvent (HTML);
-
+				
                 if (UpdateSizeInfoEvent != null)
                     UpdateSizeInfoEvent (Size, HistorySize);
             };
-
+			
             Program.Controller.FolderListChanged += delegate {
                 if (this.selected_folder != null &&
                     !Program.Controller.Folders.Contains (this.selected_folder)) {
@@ -146,35 +182,27 @@ namespace SparkleShare {
                 if (UpdateChooserEvent != null)
                     UpdateChooserEvent (Folders);
 
-                if (UpdateContentEvent != null)
-                    UpdateContentEvent (HTML);
-
                 if (UpdateSizeInfoEvent != null)
                     UpdateSizeInfoEvent (Size, HistorySize);
             };
         }
 
 
-        public static void LinkClicked (string url)
+        public void WindowClosed ()
         {
-            if (url.StartsWith (Path.VolumeSeparatorChar.ToString ())) {
+            if (HideWindowEvent != null)
+                HideWindowEvent ();
+			
+			this.selected_folder = null;
+        }
+
+
+        public void LinkClicked (string url)
+        {
+            if (url.StartsWith (Path.VolumeSeparatorChar.ToString ()) ||
+			    url.Substring (1, 1).Equals (":")) {
+				
                 Program.Controller.OpenFile (url);
-
-            } else {
-                Regex regex = new Regex (@"(.+)~(.+)~(.+)");
-                Match match = regex.Match (url);
-
-                if (match.Success) {
-                    string folder_name = match.Groups [1].Value;
-                    string revision    = match.Groups [2].Value;
-                    string note        = match.Groups [3].Value;
-
-                    Thread thread = new Thread (new ThreadStart (delegate {
-                        Program.Controller.AddNoteToFolder (folder_name, revision, note);
-                    }));
-
-                    thread.Start ();
-                }
             }
         }
     }
